/*
 * AstriMonitor.cpp
 *
 *  Created on: Jul 3, 2021
 *      Author: astrisw
 */

#include <Astri_Monitor.h>

using namespace inaf::oasbo::PacketMonitors;

std::string AstriPacketMonitor::tot_packets_key{"tot_packets"};
std::string AstriPacketMonitor::tot_MB_key{"tot_MB"};
std::string AstriPacketMonitor::tot_lost_key{"tot_lost"};
std::string AstriPacketMonitor::byte_rate_key{"byte_rate"};
std::string AstriPacketMonitor::packet_rate_key{"packet_rate"};


AstriPacketMonitor::AstriPacketMonitor() {
	reset();
}

void AstriPacketMonitor::printStats() {
	std::cout << "------------------------------" << std::endl;
	for(auto [k,v] : stats){
		if(std::string(k).compare(byte_rate_key) == 0)
			std::cout << "MByte rate: " << std::stod(v)/(1024*1024) << std::endl;
		else
			std::cout << k << ": " << v << std::endl;
	}
	std::cout << "------------------------------" << std::endl;
}

void AstriPacketMonitor::reset() {
	stats[tot_packets_key] = "0";
	stats[tot_MB_key] = "0";
	stats[tot_lost_key] = "0";
	stats[byte_rate_key] = "0";
	stats[packet_rate_key] = "0";
	for (auto [key, value] : stats) // to set to 0 type_subtype metrics
		stats[key] = "0";
	for (auto [key, value] : lastSSCMap)
		lastSSCMap[key] = 0;
	for (auto [key, value] : lostsMap)
		lostsMap[key] = 0;
    timeElps = std::chrono::steady_clock::duration::zero();
    startTime = std::chrono::steady_clock::time_point::min();
    last10sTime = std::chrono::steady_clock::now();
    bytesLast10s = 0;
    packsLast10s = 0;
}

void AstriPacketMonitor::monit(PacketLib::BasePacket &pack) {
	using namespace std;
	stats[tot_packets_key] = to_string(stoi(stats[tot_packets_key]) + 1); // count tot packets
	// counts tot for each type_subtype
	string type = to_string(pack["type"].value());
	string subtype = to_string(pack["subtype"].value());
	string key = string(type).append(subtype).append("_tot");
	stats[key] = stats.find(key) != stats.end() ? to_string(stoi(stats[key]) + 1) : "1";
	//count tot MB
	double bytes = (pack.getHeaderSize() + pack.getPayloadSize()
			+ pack.getTailSize());
	stats[tot_MB_key] =  to_string(stod(stats[tot_MB_key]) + bytes/(1024*1024));
	// update byterate each 10 seconds
    auto now = chrono::steady_clock::now();
    if (now - last10sTime > chrono::seconds(10)) {
    	chrono::duration<double> duration(now - last10sTime - chrono::seconds(10));
    	double byteRateLast10s = bytesLast10s / (duration.count()*1000);
        double packRateLast10s = packsLast10s / (duration.count()*1000);
        stats[byte_rate_key] = std::to_string(byteRateLast10s);
        stats[packet_rate_key] = std::to_string(packRateLast10s);
        last10sTime = now;
        bytesLast10s = 0;
        packsLast10s = 0;
    }
    packsLast10s+=1;
    bytesLast10s += bytes;
	// Count the number of lost packets for each type and subtype,
	// and add them up to obtain the total number of lost packets
	monitPacketLost(pack);
	int tot_losts = 0;
	for (auto [key, value] : lostsMap) {
		stats[key] = to_string(value);
		tot_losts += value;
	}
	stats[tot_lost_key] = to_string(tot_losts);
}

void AstriPacketMonitor::monitPacketLost(PacketLib::BasePacket &pack) {
	using namespace std;
	int curr_ssc = pack["ssc"].value();
	string type = to_string(pack["type"].value());
	string subtype = to_string(pack["subtype"].value());
	string last_ssc_key = string(type).append(subtype);
	string losts_key = string(last_ssc_key).append("_lost");
	int last_ssc = lastSSCMap.find(last_ssc_key) != lastSSCMap.end() ? lastSSCMap[last_ssc_key] : 0;
	int lostCount = 0;
	if (curr_ssc > last_ssc + 1) {
		lostCount = curr_ssc - last_ssc - 1;
	} else if (curr_ssc < last_ssc) {
		lostCount = 65535 - last_ssc + curr_ssc;
	}
	lastSSCMap[last_ssc_key] = curr_ssc;
	lostsMap[losts_key] =
			lostsMap.find(losts_key) != lostsMap.end() ?
					lostsMap[losts_key] + lostCount : lostCount;

}

